package dev.fyloz.colorrecipesexplorer.xlsx.component;

import dev.fyloz.colorrecipesexplorer.xlsx.builder.SheetBuilder;
import dev.fyloz.colorrecipesexplorer.xlsx.exception.InvalidCellTypeException;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbookType;
import org.slf4j.Logger;

public class Document extends XSSFWorkbook {

    public static final int WIDTH = 8;
    private static final XSSFWorkbookType WORKBOOK_TYPE = XSSFWorkbookType.XLSX;

    private final Sheet sheet;
    private final Logger logger;

    public Document(String name, Logger logger) {
        super(WORKBOOK_TYPE);
        this.logger = logger;

        sheet = new Sheet(this, createSheet(name));
    }

    public void buildSheet() {
        try {
            new SheetBuilder(sheet).build();
        } catch (InvalidCellTypeException e) {
            logger.error("Une erreur est survenue lors de la génération du document: " + e.getLocalizedMessage());
        }
    }

    public Sheet getSheet() {
        return sheet;
    }
}
