package dev.fyloz.colorrecipesexplorer.model

import dev.fyloz.colorrecipesexplorer.exception.AlreadyExistsException
import dev.fyloz.colorrecipesexplorer.exception.NotFoundException
import javax.persistence.*
import javax.validation.constraints.Min
import javax.validation.constraints.NotNull

private const val MIX_MATERIAL_DTO_MATERIAL_ID_NULL_MESSAGE = "Un identifiant de produit est requis"
private const val MIX_MATERIAL_DTO_QUANTITY_NULL_MESSAGE = "Une quantité est requise"
private const val MIX_MATERIAL_DTO_QUANTITY_NEGATIVE_MESSAGE = "La quantité ne peut pas être négative"

@Entity
@Table(name = "mix_material")
data class MixMaterial(
  @Id
  @GeneratedValue(strategy = GenerationType.IDENTITY)
  override val id: Long?,

  @ManyToOne
  @JoinColumn(name = "material_id")
  val material: Material,

  var quantity: Float,

  var position: Int
) : Model

data class MixMaterialDto(
  @field:NotNull(message = MIX_MATERIAL_DTO_MATERIAL_ID_NULL_MESSAGE)
  val materialId: Long,

  @field:NotNull(message = MIX_MATERIAL_DTO_QUANTITY_NULL_MESSAGE)
  @field:Min(value = 0, message = MIX_MATERIAL_DTO_QUANTITY_NEGATIVE_MESSAGE)
  val quantity: Float,

  val position: Int
)

// ==== DSL ====
fun mixMaterial(
  id: Long? = null,
  material: Material = material(),
  quantity: Float = 0f,
  position: Int = 0,
  op: MixMaterial.() -> Unit = {}
) = MixMaterial(id, material, quantity, position).apply(op)

fun mixMaterialDto(
  materialId: Long = 0L,
  quantity: Float = 0f,
  position: Int = 0,
  op: MixMaterialDto.() -> Unit = {}
) = MixMaterialDto(materialId, quantity, position).apply(op)

// ==== Exceptions ====
private const val MIX_MATERIAL_NOT_FOUND_EXCEPTION_TITLE = "Mix material not found"
private const val MIX_MATERIAL_ALREADY_EXISTS_EXCEPTION_TITLE = "Mix material already exists"
private const val MIX_MATERIAL_EXCEPTION_ERROR_CODE = "mixmaterial"

fun mixMaterialIdNotFoundException(id: Long) =
  NotFoundException(
    MIX_MATERIAL_EXCEPTION_ERROR_CODE,
    MIX_MATERIAL_NOT_FOUND_EXCEPTION_TITLE,
    "A mix material with the id $id could not be found",
    id
  )

fun mixMaterialIdAlreadyExistsException(id: Long) =
  AlreadyExistsException(
    MIX_MATERIAL_EXCEPTION_ERROR_CODE,
    MIX_MATERIAL_ALREADY_EXISTS_EXCEPTION_TITLE,
    "A mix material with the id $id already exists",
    id
  )
